--- sunpy/io/setup_package.py.orig	2026-07-03 08:33:41 UTC
+++ sunpy/io/setup_package.py
@@ -12,7 +12,7 @@ def get_extensions():
         return []
     cfg = defaultdict(list)
     cfg["include_dirs"].append(numpy.get_include())
-    cfg["sources"].extend(list((Path(__file__).parent / "src" / "ana").glob("*.c")))
+    cfg["sources"].extend([str(p) for p in (Path(__file__).parent / "src" / "ana").glob("*.c")])
     if get_compiler() == 'msvc':
         cfg["extra_compile_args"].extend([
             "/O2",



--- scripts/sokSettings.py.orig	2025-07-03 16:13:44 UTC
+++ scripts/sokSettings.py
@@ -16,7 +16,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-PYTHON_EXECUTABLE = "python3"
+PYTHON_EXECUTABLE = "%%PYTHON_CMD%%"
 
 from gi.repository import GLib
 



--- hid/__init__.py.orig	2025-02-13 08:24:09 UTC
+++ hid/__init__.py
@@ -8,6 +8,7 @@ library_paths = (
 
 hidapi = None
 library_paths = (
+    'libhidapi.so',
     'libhidapi-hidraw.so',
     'libhidapi-hidraw.so.0',
     'libhidapi-libusb.so',



-- Fix tests for FreeBSD (don't delete temporary files on close, fix np.int).

--- tests/test_file_attribute_manager.py.orig	2026-06-08 10:21:32 UTC
+++ tests/test_file_attribute_manager.py
@@ -12,7 +12,7 @@ class GlobalAttributeManagerTests(TestCase):
 	VALUE_IN_FILE = np.arange(3)
 
 	def setUp(self):
-		f = NamedTemporaryFile(suffix="loom")
+		f = NamedTemporaryFile(suffix="loom", delete=False)
 		f.close()
 		self.filename = f.name
 		self.file = h5py.File(f.name)



-- Fix tests for FreeBSD (don't delete temporary files on close, fix np.int).

--- tests/test_validator.py.orig	2026-06-08 10:21:32 UTC
+++ tests/test_validator.py
@@ -9,7 +9,7 @@ class LoomValidatorTests(TestCase):
 
 class LoomValidatorTests(TestCase):
     def test_file_with_empty_col_attrs_is_valid(self) -> None:
-        f = NamedTemporaryFile(suffix=".loom")
+        f = NamedTemporaryFile(suffix=".loom", delete=False)
         f.close()
         loompy.create(f.name, np.zeros((5, 5)), {}, {})
         try:



--- versioneer.py.orig	2021-05-29 06:24:51 UTC
+++ versioneer.py
@@ -339,9 +339,9 @@ def get_config_from_root(root):
     # configparser.NoOptionError (if it lacks "VCS="). See the docstring at
     # the top of versioneer.py for instructions on writing your setup.cfg .
     setup_cfg = os.path.join(root, "setup.cfg")
-    parser = configparser.SafeConfigParser()
+    parser = configparser.ConfigParser()
     with open(setup_cfg, "r") as f:
-        parser.readfp(f)
+        parser.read_file(f)
     VCS = parser.get("versioneer", "VCS")  # mandatory
 
     def get(parser, name):



--- indiweb/driver.py.orig	2021-02-24 15:36:36 UTC
+++ indiweb/driver.py
@@ -5,7 +5,7 @@ import xml.etree.ElementTree as ET
 import xml.etree.ElementTree as ET
 
 # Default INDI data directory
-INDI_DATA_DIR = os.environ.get('INDI_DATA_DIR', "/usr/share/indi/")
+INDI_DATA_DIR = os.environ.get('INDI_DATA_DIR', "%%LOCALBASE%%/share/indi/")
 
 
 class DeviceDriver:



--- locust/main.py.orig	2025-06-12 22:34:12 UTC
+++ locust/main.py
@@ -454,7 +454,7 @@ See https://github.com/locustio/locust/wiki/Installati
             sys.exit(1)
         if options.web_host == "*":
             # special check for "*" so that we're consistent with --master-bind-host
-            web_host = ""
+            web_host = "0.0.0.0"
         else:
             web_host = options.web_host
         if web_host:



--- pyradio/ping.py.orig	2025-01-18 18:10:46 UTC
+++ pyradio/ping.py
@@ -42,7 +42,7 @@ def linux_ping(server, count=1, timeout_in_seconds=1):
     '''
     try:
         r = subprocess.Popen(
-            ['ping', '-c', str(count), '-w',
+            ['ping', '-c', str(count), '-t',
              str(timeout_in_seconds), server],
             stderr=subprocess.DEVNULL,
             stdout=subprocess.PIPE



--- setup.py.orig	2025-04-18 19:16:24 UTC
+++ setup.py
@@ -7,6 +7,8 @@ try:
 
 try:
     import multiprocessing
+    # Python 3.14 uses forkserver, which fails. Force spawn.
+    multiprocessing.set_start_method('spawn', force=True)
 except ImportError:
     multiprocessing = None
 
@@ -233,5 +235,5 @@ def long_desc_from_readme():
         long_description = re.compile(r"^\.\. highlight:: \w+$", re.M).sub("", long_description)
         return long_description
 
-
-setup(cmdclass=cmdclass, ext_modules=ext_modules, long_description=long_desc_from_readme())
+if __name__ == '__main__':
+    setup(cmdclass=cmdclass, ext_modules=ext_modules, long_description=long_desc_from_readme())



-- Add setup.py to define extension modules.
-- This is needed because the current version of setuptools on FreeBSD does not
-- support defining ext-modules in pyproject.toml.

--- setup.py.orig	2026-05-29 15:20:01 UTC
+++ setup.py
@@ -0,0 +1,9 @@
+from setuptools import setup, Extension
+
+setup(
+    ext_modules=[
+        Extension("hits.adapters_cython", ["hits/adapters_cython.pyx"]),
+        Extension("hits.fastq_cython", ["hits/fastq_cython.pyx"]),
+        Extension("hits.sw_cython", ["hits/sw_cython.pyx"]),
+    ],
+)



--- asrun/common/sysutils.py.orig	2017-07-13 07:40:52 UTC
+++ asrun/common/sysutils.py
@@ -83,7 +83,7 @@ def on_mac():
 
 def on_linux():
     """Tell if it's running on a linux platform"""
-    return not on_windows()
+    return sys.platform in ("linux")
 
 def on_64bits():
     """Tell if it's running on a 64 bits platform"""
@@ -212,7 +212,7 @@ class FileName(object):
         and fill properties = user, passwd, mach, path
         """
         # windows : only work on local files
-        if not on_linux():
+        if on_windows():
             self.path = pathname
             return
         # posix



#!/usr/bin/env python3
"""Report outdated FreeBSD ports for a maintainer using Repology and GitHub."""

# report-outdated-for-maintainer.py - a script that find outdated ports for
# a given maintainer based on Repology API and GitHub releases, and prints
# a categorized report.
#
# MAINTAINER= yuri@FreeBSD.org
#


import sys
import os
import re
import json
import time
import subprocess
import concurrent.futures
import urllib.request
import urllib.parse
import urllib.error
import ssl

# --- Dependency check ---

missing_deps = []
try:
    from tabulate2 import tabulate
except ImportError:
    missing_deps.append("tabulate2")

if missing_deps:
    py_ver = f"py{sys.version_info.major}{sys.version_info.minor}"
    pkg_names = " ".join(f"{py_ver}-{dep}" for dep in missing_deps)
    print(
        f"Missing dependencies: {', '.join(missing_deps)}. "
        f"Please install them with 'pkg install {pkg_names}' and try again."
    )
    sys.exit(1)

# --- Configuration ---

PORTSDIR = os.environ.get("PORTSDIR", "/usr/ports")
REPOLOGY_API_BASE = "https://repology.org/api/v1"
GITHUB_BASE = "https://github.com"
GITHUB_API_BASE = "https://api.github.com"
USER_AGENT = "report-outdated-for-maintainer/1.0 (FreeBSD ports maintainer tool)"
GITHUB_TOKEN = None

# USES keyword → Port Type label (in priority order)
USES_TYPES = [
    ("gmake",    "gmake"),
    ("cmake",    "cmake"),
    ("meson",    "meson"),
    ("waf",      "waf"),
    ("python",   "Python"),
    ("cargo",    "Rust"),
    ("cabal",    "Haskel"),
    ("go",       "GoLang"),
    ("nodejs",   "NodeJS"),
    ("fortran",  "Fortran"),
    ("drupal",   "Drupal"),
    ("cran",     "R"),
    ("ocaml",    "Ocaml"),
    ("php",      "php"),
    ("vala",     "Vala"),
    ("ruby",     "Ruby"),
    ("perl5",    "Perl"),
    ("mono",     "Mono"),
    ("ada",      "Ada"),
    ("lazarus",  "Lazarus"),
    ("erlang",   "Erlang"),
    ("electron", "Electron"),
    ("linux",    "Linux"),
    ("kmod",     "kmod"),
    ("java",     "Java"),
]


# --- Validation ---

def check_portsdir():
    required = [
        PORTSDIR,
        os.path.join(PORTSDIR, "Makefile"),
        os.path.join(PORTSDIR, "Mk"),
        os.path.join(PORTSDIR, "MOVED"),
        os.path.join(PORTSDIR, "devel"),
        os.path.join(PORTSDIR, "devel", "Makefile"),
    ]
    for path in required:
        if not os.path.exists(path):
            print(f"Error: Required path not found: {path}", file=sys.stderr)
            sys.exit(1)


def get_github_token():
    """Return a GitHub token from env or gh auth, if available."""
    global GITHUB_TOKEN
    if GITHUB_TOKEN is not None:
        return GITHUB_TOKEN

    GITHUB_TOKEN = os.environ.get("GITHUB_TOKEN") or os.environ.get("GH_TOKEN")
    if GITHUB_TOKEN:
        return GITHUB_TOKEN

    try:
        result = subprocess.run(
            ["gh", "auth", "token"],
            capture_output=True,
            text=True,
            timeout=10,
        )
        token = result.stdout.strip()
        if result.returncode == 0 and token:
            GITHUB_TOKEN = token
            return GITHUB_TOKEN
    except Exception:
        pass

    GITHUB_TOKEN = ""
    return None


# --- Ports tree parsing ---

def get_subdir_entries(makefile_path):
    """Return list of SUBDIR += entries from a Makefile."""
    entries = []
    try:
        with open(makefile_path, "r", errors="replace") as f:
            content = f.read()
        for m in re.finditer(r"^\s*SUBDIR\s*\+=\s*(\S+)", content, re.MULTILINE):
            entries.append(m.group(1))
    except OSError:
        pass
    return entries


def read_makefile_logical_lines(makefile_path):
    """Read a Makefile and return a list of logical lines (continuations joined)."""
    try:
        with open(makefile_path, "r", errors="replace") as f:
            content = f.read()
    except OSError:
        return []
    # Join continuation lines
    content = re.sub(r"\\\n", " ", content)
    return content.splitlines()


def get_makefile_var(makefile_path, var):
    """Extract a simple variable value from a Makefile (first assignment found)."""
    for line in read_makefile_logical_lines(makefile_path):
        m = re.match(rf"^\s*{re.escape(var)}\s*(?::|\?)?=\s*(.+)$", line)
        if m:
            return re.sub(r"\s+#.*$", "", m.group(1)).strip()
    return None


def get_uses_types(makefile_path):
    """Return a comma-separated string of port types based on USES= in the Makefile."""
    uses_tokens = set()
    for line in read_makefile_logical_lines(makefile_path):
        # Match both USES= and USES+=
        m = re.match(r"^\s*USES\s*\+?=\s*(.+)$", line)
        if m:
            for token in m.group(1).split():
                # Strip :options suffix for matching
                uses_tokens.add(token.split(":")[0])

    types = []
    for keyword, label in USES_TYPES:
        if keyword in uses_tokens:
            types.append(label)
    return ", ".join(types)


def resolve_make_value(value, portname):
    """Resolve simple ${PORTNAME} substitutions in Makefile values."""
    if value is None:
        return None

    def replacer(match):
        modifier = match.group(1)
        if modifier == ":tl":
            return portname.lower()
        if modifier == ":tu":
            return portname.upper()
        return portname

    return re.sub(r"\$\{PORTNAME(?:(:tl|:tu))?\}", replacer, value)


def get_github_repo(makefile_path):
    """Return (account, project) for USE_GITHUB=yes ports or from WWW field, else None."""
    use_github = get_makefile_var(makefile_path, "USE_GITHUB")
    if use_github and use_github.lower() == "yes":
        portname = get_makefile_var(makefile_path, "PORTNAME")
        if not portname:
            return None

        gh_account = get_makefile_var(makefile_path, "GH_ACCOUNT") or portname
        gh_project = get_makefile_var(makefile_path, "GH_PROJECT") or portname
        gh_account = resolve_make_value(gh_account.split()[0], portname)
        gh_project = resolve_make_value(gh_project.split()[0], portname)
        return gh_account, gh_project
    
    # If USE_GITHUB is not set, try to extract from WWW field, but only for ports
    # that appear to use GitHub releases and are explicitly marked with a comment
    # or have DISTVERSION matching a GitHub-style tag pattern (YYYY.M.DD or similar)
    version = get_makefile_var(makefile_path, "DISTVERSION") or get_makefile_var(makefile_path, "PORTVERSION")
    if version and looks_like_version(version):
        # Only query from WWW if version matches GitHub-style patterns:
        # - YYYY.M.DD (date-based like 2026.4.26, 2026.4.29)
        # - X.Y.Z.W (4+ numeric components)
        # This filters out most non-GitHub ports while catching real GitHub releases
        parts = re.findall(r"\d+", version)
        if len(parts) >= 3:
            # Check if it looks like YYYY.M.DD (year >= 2000)
            if int(parts[0]) >= 2000 and int(parts[0]) <= 2099:
                if int(parts[1]) >= 1 and int(parts[1]) <= 12:
                    if int(parts[2]) >= 1 and int(parts[2]) <= 31:
                        # Looks like a date-based version from GitHub
                        www = get_makefile_var(makefile_path, "WWW")
                        if www:
                            m = re.match(r"https?://(?:www\.)?github\.com/([^/]+)/([^/\s]+)", www)
                            if m:
                                return m.group(1), m.group(2)
            # Also check for simple 4+ component versions (e.g., 1.2.3.4)
            elif len(parts) >= 4:
                www = get_makefile_var(makefile_path, "WWW")
                if www:
                    m = re.match(r"https?://(?:www\.)?github\.com/([^/]+)/([^/\s]+)", www)
                    if m:
                        return m.group(1), m.group(2)
    
    return None


# --- Version handling ---

VERSION_RE = re.compile(r"^[A-Za-z0-9][A-Za-z0-9._+-]*$")
G_VERSION_RE = re.compile(r"^g[a-zA-Z0-9]+$")
YYYYMMDD_RE = re.compile(r"^(\d{4})(\d{2})(\d{2})$")


def looks_like_version(s):
    return bool(s and VERSION_RE.match(s) and any(ch.isdigit() for ch in s))


def compute_aux_version(real_version, latest_effective=None):
    """
    Return the auxiliary version string if it differs from real_version, else None.

    - gNNNNN  → 0.0.0... (matching component count of latest_effective)
    - YYYYMMDD → YYYY.MM.DD
    """
    if G_VERSION_RE.match(real_version) and latest_effective is not None:
        parts = parse_version(latest_effective)
        n = max(len(parts), 1)
        return ".".join(["0"] * n)

    m = YYYYMMDD_RE.match(real_version)
    if m:
        year, month, day = int(m.group(1)), int(m.group(2)), int(m.group(3))
        if 1900 <= year <= 2099 and 1 <= month <= 12 and 1 <= day <= 31:
            return f"{m.group(1)}.{m.group(2)}.{m.group(3)}"

    return None


def display_version(real, aux):
    """Return 'real (aux)' if aux differs from real, else just 'real'."""
    if aux and aux != real:
        return f"{real} ({aux})"
    return real


def parse_version(version_str):
    """Parse version string into a tuple of ints."""
    parts = [int(part) for part in re.findall(r"\d+", version_str)]
    return tuple(parts) if parts else (0,)


def compare_versions(left_version, right_version):
    """Return 1 if left>right, -1 if left<right, 0 if equal by numeric components."""
    left = parse_version(left_version)
    right = parse_version(right_version)
    max_len = max(len(left), len(right), 3)
    left = left + (0,) * (max_len - len(left))
    right = right + (0,) * (max_len - len(right))
    if left > right:
        return 1
    if left < right:
        return -1
    return 0


def categorize_outdatedness(current_effective, latest_effective):
    """
    Compare effective (auxiliary) versions.
    Returns (category, level) or None if not outdated.
    """
    if compare_versions(latest_effective, current_effective) <= 0:
        return None

    cur = parse_version(current_effective)
    lat = parse_version(latest_effective)
    max_len = max(len(cur), len(lat), 3)
    cur = cur + (0,) * (max_len - len(cur))
    lat = lat + (0,) * (max_len - len(lat))

    if lat[0] > cur[0]:
        return ("major", lat[0] - cur[0])

    if lat[1] > cur[1]:
        return ("minor", lat[1] - cur[1])

    for i in range(2, max_len):
        if lat[i] > cur[i]:
            return ("patch", lat[i] - cur[i])
        if lat[i] < cur[i]:
            break

    return ("patch", 1)


# --- Ports scanning ---

def get_port_version_from_makefile(makefile_path):
    """Try PORTVERSION then DISTVERSION; return value if it looks like a version."""
    for var in ("PORTVERSION", "DISTVERSION"):
        val = get_makefile_var(makefile_path, var)
        if val and looks_like_version(val):
            return val
    return None


def get_port_version_via_make(port_dir):
    """Run 'make -V PORTVERSION' in port_dir and return the result."""
    try:
        result = subprocess.run(
            ["make", "BATCH=yes", "-V", "PORTVERSION"],
            cwd=port_dir,
            capture_output=True,
            text=True,
            timeout=30,
        )
        v = result.stdout.strip()
        if looks_like_version(v):
            return v
    except Exception:
        pass
    return None


def find_maintained_ports(maintainer_email):
    """
    Return (versions, uses_types, github_repos, github_version_prefixes) keyed by port origin.
    versions[origin] = real version string
    uses_types[origin] = comma-separated port type string
    github_repos[origin] = (account, project) for USE_GITHUB ports
    github_version_prefixes[origin] = (prefix, suffix) to strip from GitHub release tags
    """
    versions = {}
    uses_types = {}
    github_repos = {}
    github_version_prefixes = {}
    top_makefile = os.path.join(PORTSDIR, "Makefile")
    categories = get_subdir_entries(top_makefile)

    for category in categories:
        cat_makefile = os.path.join(PORTSDIR, category, "Makefile")
        if not os.path.isfile(cat_makefile):
            continue
        port_dirs = get_subdir_entries(cat_makefile)
        for port_dir in port_dirs:
            port_path = os.path.join(PORTSDIR, category, port_dir)
            port_makefile = os.path.join(port_path, "Makefile")
            if not os.path.isfile(port_makefile):
                continue
            maintainer = get_makefile_var(port_makefile, "MAINTAINER")
            if not maintainer or maintainer.lower() != maintainer_email.lower():
                continue
            origin = f"{category}/{port_dir}"
            version = get_port_version_from_makefile(port_makefile)
            if version is None:
                version = get_port_version_via_make(port_path)
            if version is None:
                print(
                    f"Warning: Could not determine version for {origin}, skipping.",
                    file=sys.stderr,
                )
                continue
            versions[origin] = version
            uses_types[origin] = get_uses_types(port_makefile)
            github_repo = get_github_repo(port_makefile)
            if github_repo:
                github_repos[origin] = github_repo
                portname = get_makefile_var(port_makefile, "PORTNAME") or ""
                prefix_raw = get_makefile_var(port_makefile, "DISTVERSIONPREFIX") or ""
                suffix_raw = get_makefile_var(port_makefile, "DISTVERSIONSUFFIX") or ""
                prefix = resolve_make_value(prefix_raw, portname)
                suffix = resolve_make_value(suffix_raw, portname)
                if prefix or suffix:
                    github_version_prefixes[origin] = (prefix, suffix)

    return versions, uses_types, github_repos, github_version_prefixes


# --- Repology API ---

def repology_request(url):
    """Make a single GET request to the Repology API; return parsed JSON."""
    ctx = ssl.create_default_context()
    req = urllib.request.Request(url, headers={"User-Agent": USER_AGENT})
    try:
        with urllib.request.urlopen(req, timeout=30, context=ctx) as resp:
            return json.loads(resp.read())
    except urllib.error.HTTPError as e:
        print(f"Error: Repology API returned HTTP {e.code}: {e.reason}", file=sys.stderr)
        sys.exit(1)
    except urllib.error.URLError as e:
        print(f"Error: Could not reach Repology API: {e.reason}", file=sys.stderr)
        sys.exit(1)
    except Exception as e:
        print(f"Error querying Repology: {e}", file=sys.stderr)
        sys.exit(1)


def query_repology_outdated(maintainer_email):
    """
    Return dict {origin: latest_version} for FreeBSD ports that Repology
    considers outdated for this maintainer.
    """
    encoded_email = urllib.parse.quote(maintainer_email.lower(), safe="@")
    base_params = f"maintainer={encoded_email}&inrepo=freebsd&outdated=true"
    url = f"{REPOLOGY_API_BASE}/projects/?{base_params}"
    outdated = {}

    while url:
        data = repology_request(url)
        if not data:
            break

        for project_name, packages in data.items():
            freebsd_outdated = None
            newest_version = None

            for pkg in packages:
                if pkg.get("repo") == "freebsd" and pkg.get("status") == "outdated":
                    freebsd_outdated = pkg
                if pkg.get("status") == "newest" and newest_version is None:
                    newest_version = pkg.get("version")

            if freebsd_outdated and newest_version:
                srcname = freebsd_outdated.get("srcname") or freebsd_outdated.get("visiblename")
                if srcname:
                    outdated[srcname] = newest_version

        if len(data) == 200:
            last_project = list(data.keys())[-1]
            encoded_last = urllib.parse.quote(last_project, safe="")
            url = f"{REPOLOGY_API_BASE}/projects/{encoded_last}/?{base_params}"
            time.sleep(0.5)  # be polite to the API
        else:
            url = None

    return outdated


def github_request(url, method="GET"):
    """Make a single GitHub request and return the response object."""
    headers = {
        "User-Agent": USER_AGENT,
        "Accept": "application/vnd.github+json",
    }
    token = get_github_token()
    if token:
        headers["Authorization"] = f"Bearer {token}"
    req = urllib.request.Request(url, headers=headers, method=method)
    return urllib.request.urlopen(req, timeout=30)


def query_github_latest_release(account, project, cache):
    """Return the latest stable GitHub release tag for account/project, or None."""
    key = (account, project)
    if key in cache:
        return cache[key]

    # Use 'gh' CLI for better rate limit handling and authentication
    try:
        result = subprocess.run(
            ["gh", "release", "view", "--repo", f"{account}/{project}"],
            capture_output=True,
            text=True,
            timeout=30,
        )
        if result.returncode == 0:
            for line in result.stdout.split("\n"):
                if line.startswith("tag:"):
                    tag = line.split(":", 1)[1].strip()
                    cache[key] = tag.strip() or None
                    return cache[key]
        cache[key] = None
        return None
    except Exception:
        cache[key] = None
        return None


def query_github_releases(github_repos, version_prefixes=None):
    """Return dict {origin: latest_release_tag} for GitHub-based ports with releases."""
    cache = {}
    latest_releases = {}

    unique_repos = sorted(set(github_repos.values()))
    # gh CLI handles rate limiting better than direct API calls, so we can use more workers
    max_workers = min(8, len(unique_repos)) if unique_repos else 1
    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
        future_map = {
            executor.submit(query_github_latest_release, account, project, cache): (account, project)
            for account, project in unique_repos
        }
        for future in concurrent.futures.as_completed(future_map):
            future.result()

    for origin, repo in github_repos.items():
        latest = cache.get(repo)
        if latest:
            if version_prefixes and origin in version_prefixes:
                prefix, suffix = version_prefixes[origin]
                if prefix and latest.startswith(prefix):
                    latest = latest[len(prefix):]
                if suffix and latest.endswith(suffix):
                    latest = latest[:-len(suffix)]
            latest_releases[origin] = latest
    return latest_releases


# --- Report formatting ---

SEP = "============="


def format_table_with_blank_header_line(rows, headers):
    """Render tabulate 'simple' table with a blank line inserted after the header separator."""
    raw = tabulate(rows, headers=headers, tablefmt="simple")
    lines = raw.splitlines()
    # simple format: line 0 = header names, line 1 = dashes, line 2+ = data
    if len(lines) >= 2:
        lines.insert(2, "")
    return "\n".join(lines)


def choose_latest_version(repology_real, github_real):
    """Choose the highest latest version from Repology/GitHub and return display info."""
    candidates = []
    if repology_real:
        repology_aux = compute_aux_version(repology_real)
        repology_effective = repology_aux if repology_aux else repology_real
        candidates.append(("Repology", repology_real, repology_aux, repology_effective))
    if github_real:
        github_aux = compute_aux_version(github_real)
        github_effective = github_aux if github_aux else github_real
        candidates.append(("GitHub", github_real, github_aux, github_effective))

    if not candidates:
        return None

    best = candidates[0]
    for candidate in candidates[1:]:
        cmp = compare_versions(candidate[3], best[3])
        if cmp > 0 or (cmp == 0 and candidate[0] == "Repology" and best[0] != "Repology"):
            best = candidate

    notes = [candidate[0] for candidate in candidates if compare_versions(candidate[3], best[3]) == 0]
    return {
        "real": best[1],
        "aux": best[2],
        "effective": best[3],
        "notes": ", ".join(notes),
    }


def print_report(maintainer_email, local_versions, local_uses, outdated_repology, github_latest):
    total_ports = len(local_versions)

    headers = [
        "Port origin",
        "Latest released version",
        "Current port version",
        "Outdatedness level",
        "Port Type",
        "Notes",
    ]

    major_ports = []
    minor_ports = []
    patch_ports = []

    for origin, local_real in local_versions.items():
        latest = choose_latest_version(outdated_repology.get(origin), github_latest.get(origin))
        if latest is None:
            continue

        # Compute auxiliary version for local (gNNNNN needs latest_effective for depth)
        local_aux = compute_aux_version(local_real, latest["effective"])
        local_effective = local_aux if local_aux else local_real

        result = categorize_outdatedness(local_effective, latest["effective"])
        if result is None:
            continue
        category, level = result

        port_type = local_uses.get(origin, "")
        row = (
            origin,
            display_version(latest["real"], latest["aux"]),
            display_version(local_real, local_aux),
            level,
            port_type,
            latest["notes"],
        )
        if category == "major":
            major_ports.append(row)
        elif category == "minor":
            minor_ports.append(row)
        else:
            patch_ports.append(row)

    def sort_key(row):
        return (-row[3], row[0])

    major_ports.sort(key=sort_key)
    minor_ports.sort(key=sort_key)
    patch_ports.sort(key=sort_key)

    n_major = len(major_ports)
    n_minor = len(minor_ports)
    n_patch = len(patch_ports)
    n_outdated = n_major + n_minor + n_patch

    # Title
    print(f"Outdated ports for maintainer {maintainer_email}")
    print()
    print()

    # Summary line
    pct = (lambda n: f"{n / total_ports * 100:.2f}%") if total_ports > 0 else (lambda n: "0.00%")

    print(
        f"The maintainer has {n_outdated} ({pct(n_outdated)}) outdated ports, categorized as follows: "
        f"{n_major} ({pct(n_major)}) majorly outdated, "
        f"{n_minor} ({pct(n_minor)}) moderately outdated, "
        f"and {n_patch} ({pct(n_patch)}) slightly outdated."
    )
    print()

    for title, rows in [
        ("Majorly outdated ports", major_ports),
        ("Moderately outdated ports", minor_ports),
        ("Slightly outdated ports", patch_ports),
    ]:
        print(f"{SEP}{title}{SEP}")
        print()
        if rows:
            print(format_table_with_blank_header_line(rows, headers))
        else:
            print("No ports in this category")
        print()


# --- Main ---

def main():
    if len(sys.argv) != 2:
        print(f"Usage: {sys.argv[0]} <maintainer-email>", file=sys.stderr)
        sys.exit(1)

    maintainer_email = sys.argv[1]
    check_portsdir()

    print(f"Scanning {PORTSDIR} for ports maintained by {maintainer_email}...", file=sys.stderr)
    local_versions, local_uses, github_repos, github_version_prefixes = find_maintained_ports(maintainer_email)

    if not local_versions:
        print(f"No ports maintained by {maintainer_email}")
        sys.exit(0)

    print(
        f"Found {len(local_versions)} local ports ({len(github_repos)} GitHub-based). Querying Repology and GitHub...",
        file=sys.stderr,
    )
    outdated_repology = query_repology_outdated(maintainer_email)
    github_latest = query_github_releases(github_repos, github_version_prefixes)
    print(f"Repology reports {len(outdated_repology)} outdated FreeBSD ports.", file=sys.stderr)
    print(f"GitHub provides latest releases for {len(github_latest)} GitHub-based ports.", file=sys.stderr)

    print_report(maintainer_email, local_versions, local_uses, outdated_repology, github_latest)


if __name__ == "__main__":
    main()



-- Exclude tests from installation.

--- setup.py.orig	2026-06-11 07:14:15 UTC
+++ setup.py
@@ -42,10 +42,6 @@ DATA_FILES = [os.path.relpath(file_name, "vunit") for 
 setup(
     name="vunit_hdl",
     version=version(),
     packages=[
-        "tests",
-        "tests.lint",
-        "tests.unit",
-        "tests.acceptance",
         "vunit",
         "vunit.com",
         "vunit.parsing",



--- setup.py.orig	2026-04-11 20:15:35 UTC
+++ setup.py
@@ -160,7 +160,7 @@ def clean_before_build(command):
     if command in ["build", "build_ext", "clean", "sdist"]:
         print("removing __pycache__ directories recursively")
         subprocess.check_call(
-         ['/bin/bash', '-c', "find . -name '__pycache__*' -prune | xargs rm -rf"])
+         ['/bin/sh', '-c', "find . -name '__pycache__*' -prune | xargs rm -rf"])
 
     # Symlinked extension libraries trip up "setup.py sdist". Delete them.
     if command in ["clean", "sdist"]:
@@ -313,8 +313,7 @@ class Extension_lm(Extension):
                            libraries = [],
                            define_macros=[('NDEBUG', '1')],
                            extra_compile_args=[
-                               "-Wsign-compare",
-                               "-Wlogical-op"],
+                               "-Wsign-compare"],
                           )
 
 extension_lm = Extension_lm("Onboard", "Onboard")
@@ -401,7 +400,7 @@ class build_i18n_custom(DistUtilsExtra.auto.build_i18n
             # Get the autostart directory
             autostart_destination = os.path.join(config_path, "autostart") 
         else:
-            autostart_destination = '/etc/xdg/autostart'
+            autostart_destination = 'etc/xdg/autostart'
   
 
         for i, file_set in enumerate(self.distribution.data_files):
@@ -439,33 +438,6 @@ class CustomInstallCommand(install):
         # Run the default installation
         install.run(self)
 
-        # Only run this if NOT inside a fakeroot environment
-        if not os.getenv("FAKEROOTKEY"):
-            print("Running tools/gen_gschema.py...")
-            
-
-
-            # Correct install base from setuptools
-            install_base = Path(self.install_data)
-            schema_dir = install_base / "share" / "glib-2.0" / "schemas"
-                
-
-            # Ensure the schema directory exists
-            schema_dir.mkdir(parents=True, exist_ok=True)
-            
-            print("Running glib-compile-schemas...")
-
-            try:
-                if os.path.exists(schema_dir):
-                    subprocess.check_call(["glib-compile-schemas", schema_dir])
-                else:
-                    print(f"Warning: Schema directory not found: {schema_dir}")
-            except subprocess.CalledProcessError as e:
-                print(f"Error running glib-compile-schemas: {e}")
-                sys.exit(1)
-        else:
-            print("Skipping tools/gen_gschema.py and glib-compile-schemas since this is a fakeroot environment.")
-
 class UninstallCommand(Command):
     """Custom uninstall command to remove all installed files"""
 
@@ -621,16 +593,8 @@ DistUtilsExtra.auto.setup(
     description = 'Simple On-screen Keyboard',
 
     packages = ['Onboard', 'Onboard.pypredict'],
-    data_files = [('share/glib-2.0/schemas', glob.glob('data/*.gschema.xml')),
+    data_files = [('share/glib-2.0/schemas', glob.glob('data/org.onboard.gschema.xml')),
                   ('share/dbus-1/services', glob.glob('data/org.onboard.Onboard.service')),
-                  ('share/doc/onboard', glob.glob('AUTHORS')),
-                  ('share/doc/onboard', glob.glob('CHANGELOG')),
-                  ('share/doc/onboard', glob.glob('COPYING*')),
-                  ('share/doc/onboard', glob.glob('HACKING')),
-                  ('share/doc/onboard', glob.glob('DBUS.md')),
-                  ('share/doc/onboard', glob.glob('README.md')),
-                  ('share/doc/onboard', glob.glob('onboard-defaults.conf.example')),
-                  ('share/doc/onboard', glob.glob('onboard-default-settings.gschema.override.example')),
                   ('share/icons/hicolor/16x16/apps', glob.glob('icons/hicolor/16/*')),
                   ('share/icons/hicolor/22x22/apps', glob.glob('icons/hicolor/22/*')),
                   ('share/icons/hicolor/24x24/apps', glob.glob('icons/hicolor/24/*')),
@@ -650,17 +614,13 @@ DistUtilsExtra.auto.setup(
                   ('share/onboard/models', glob.glob('models/*.lm')),
                   ('share/onboard/tools', glob.glob('Onboard/pypredict/tools/checkmodels')),
                   ('share/onboard/emojione/svg', glob.glob('emojione/svg/*.svg')),
-                  ('share/gnome-shell/extensions/Onboard_Indicator@onboard.org',
-                      glob_files('gnome/{}/Onboard_Indicator@onboard.org/*'.format(gnome_shell_version))),
-                  ('share/gnome-shell/extensions/Onboard_Indicator@onboard.org/schemas',
-                      glob_files('gnome/{}/Onboard_Indicator@onboard.org/schemas/*'.format(gnome_shell_version))),
                  ],
 
     scripts = ['onboard', 'onboard-settings'],
 
     options={
         'build_scripts': {
-            'executable': '/usr/bin/python3'
+            'executable': '%%PYTHON_CMD%%'
         }
     },
     



#!/usr/bin/env python
#
# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2023 Charlie Li <vishwin@FreeBSD.org>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# MAINTAINER=	python@FreeBSD.org

import argparse
import csv

argparser = argparse.ArgumentParser(description="Strip other columns from RECORD.")
argparser.add_argument('file', type=open, help="path to RECORD file")
args = argparser.parse_args()

csvreader = csv.reader(args.file)
for row in csvreader:
        print(row[0])



--- waflib/Context.py.orig	2026-06-27 09:53:00.227806000 +0200
+++ waflib/Context.py	2026-06-27 09:53:18.691344000 +0200
@@ -2,9 +2,17 @@
 # encoding: utf-8
 # WARNING! Do not edit! https://waf.io/book/index.html#_obtaining_the_waf_file
 
-import os,re,imp,sys
+import os,re,sys
 from waflib import Utils,Errors,Logs
 import waflib.Node
+
+if sys.hexversion > 0x3040000:
+    import types
+    class imp(object):
+        new_module = lambda x: types.ModuleType(x)
+else:
+    import imp
+
 HEXVERSION=0x2000e00
 WAFVERSION="2.0.14"
 WAFREVISION="907519cab9c1c8c7e4f7d4e468ed6200b9250d58"



--- Onboard/Indicator.py.orig	2025-07-03 16:13:44 UTC
+++ Onboard/Indicator.py
@@ -133,7 +133,7 @@ class ContextMenu(GObject.GObject):
         self._keyboard.request_visibility_toggle()
 
     def _on_help(self, data=None):
-        subprocess.Popen(["/usr/bin/yelp", "help:onboard"])
+        subprocess.Popen(["%%LOCALBASE%%/bin/yelp", "help:onboard"])
 
     def _on_quit(self, data=None):
         _logger.debug("Entered _on_quit")



#!/usr/bin/env python
# ex:ts=4
#-*- mode: Fundamental; tab-width: 4; -*-
#
# patchtool.py - a tool to automate common operation with patchfiles in the
# FreeBSD Ports Collection.
#
# ----------------------------------------------------------------------------
# "THE BEER-WARE LICENSE" (Revision 42, (c) Poul-Henning Kamp):
# Maxim Sobolev <sobomax@FreeBSD.org> wrote this file.  As long as you retain
# this notice you can do whatever you want with this stuff. If we meet some
# day, and you think this stuff is worth it, you can buy me a beer in return.
#
# Maxim Sobolev
# ----------------------------------------------------------------------------
#
# MAINTAINER= ports@FreeBSD.org <- any unapproved commits to this file are
#				     highly discouraged!!!
#

import os, os.path, subprocess, sys, getopt, glob, errno, types

# python3 lacks raw_input
compat_raw_input = input
if sys.version_info < (3,):
    compat_raw_input = raw_input

# Some global variables used as constants
#True = 1
#False = 0

def isStr(obj):
    try:
        return isinstance(obj, basestring)
    except NameError:
        return isinstance(obj, str)

# Tweakable global variables. User is able to override any of these by setting
# appropriate environment variable prefixed by `PT_', eg:
# $ export PT_CVS_ID="FooOS"
# $ export PT_DIFF_CMD="/usr/local/bin/mydiff"
# will force script to use "FooOS" as a CVS_ID instead of "FreeBSD" and
# "/usr/local/bin/mydiff" as a command to generate diffs.
class Vars:
	CVS_ID = 'FreeBSD'

	DIFF_ARGS = '-du'
	DIFF_SUFX = '.orig'
	PATCH_PREFIX = 'patch-'
	PATCH_IGN_SUFX = ('.orig', '.rej')
	RCSDIFF_SUFX = ',v'

	CD_CMD = 'cd'
	DIFF_CMD = '/usr/bin/diff'
	MAKE_CMD = '/usr/bin/make'
	PRINTF_CMD = '/usr/bin/printf'
	RCSDIFF_CMD = '/usr/bin/rcsdiff'

	DEFAULT_MAKEFILE = 'Makefile'
	DEV_NULL = '/dev/null'
	ETC_MAKE_CONF = '/etc/make.conf'
	
	SLASH_REPL_SYMBOL = '_'	# The symbol to replace '/' when auto-generating
							# patchnames


#
# Check if the supplied patch refers to a port's directory.
#
def isportdir(path, soft = False):
	REQ_FILES = ('Makefile', 'pkg-descr', 'distinfo')
	if not os.path.isdir(path) and soft != True:
		raise IOError(errno.ENOENT, path)
		# Not reached #

	try:
		content = os.listdir(path)
	except OSError:
		return False

	for file in REQ_FILES:
		if file not in content:
			return False
	return True


#
# Traverse directory tree up from the path pointed by argument and return if
# root directory of a port is found.
#
def locateportdir(path, wrkdirprefix= '', strict = False):
	# Flag to relax error checking in isportdir() function. It required when
	# WRKDIRPREFIX is defined.
	softisport = False

	path = os.path.abspath(path)

	if wrkdirprefix != '':
		wrkdirprefix= os.path.abspath(wrkdirprefix)
		commonprefix = os.path.commonprefix((path, wrkdirprefix))
		if commonprefix != wrkdirprefix:
			return ''
		path = path[len(wrkdirprefix):]
		softisport = True

	while path != '/':
		if isportdir(path, softisport) == True:
			return path
		path = os.path.abspath(os.path.join(path, '..'))

	if strict == True:
		raise LocatePDirError(path)
		# Not reached #
	else:
		return ''


#
# Get value of a make(1) variable called varname. Optionally maintain a cache
# for resolved varname:makepath pairs to speed-up operation if the same variable
# from the exactly same file is requested repeatedly (invocation of make(1) is
# very expensive operation...)
#
def querymakevar(varname, path = 'Makefile', strict = False, cache = {}):
	path = os.path.abspath(path)

	if (varname, path) in cache:
		return cache[(varname, path)]

	origpath = path
	if os.path.isdir(path):
		path = os.path.join(path, Vars.DEFAULT_MAKEFILE)
	if not os.path.isfile(path):
		raise IOError(errno.ENOENT, path)
		# Not reached #

	dir = os.path.dirname(path)
	CMDLINE = '%s %s && %s -f %s -V %s' % (Vars.CD_CMD, dir, Vars.MAKE_CMD, \
      path, varname)
	devnull = open('/dev/null', 'a')
	pipe = subprocess.Popen(CMDLINE, shell = True, stdin = subprocess.PIPE, \
            stdout = subprocess.PIPE, stderr = devnull, close_fds = True)
	retval = ''
	for line in pipe.stdout.readlines():
		retval = retval + line.decode().strip() + ' '
	retval = retval[:-1]
	if strict == True and retval.strip() == '':
		raise MakeVarError(path, varname)
		# Not reached #

	cache[(varname, origpath)] = retval
	return retval


#
# Get a path of `path'  relatively to wrksrc. For example:
# path:		/foo/bar
# wrksrc:	/foo/bar/baz/somefile.c
# getrelpath:	baz/somefile.c
# Most of the code here is to handle cases when ../ operation is required to
# reach wrksrc from path, for example:
# path:		/foo/bar
# wrksrc:	/foo/baz/somefile.c
# getrelpath:	../baz/somefile.c
#
def getrelpath(path, wrksrc):
	path = os.path.abspath(path)
	wrksrc = os.path.abspath(wrksrc) + '/'
	commonpart = os.path.commonprefix((path, wrksrc))
	while commonpart[-1:] != '/':
		commonpart = commonpart[:-1]
	path = path[len(commonpart):]
	wrksrc = wrksrc[len(commonpart):]
	adjust = ''
	while os.path.normpath(os.path.join(wrksrc, adjust)) != '.':
		adjust = os.path.join(adjust, '..')
	relpath = os.path.join(adjust, path)
	return relpath


#
# Generate a diff between saved and current versions of the file pointed by the
# wrksrc+path. Apply heuristics to locate saved version of the file in question
# and if it fails assume that file is new, so /dev/null is to be used as
# original file. Optionally save generated patch into `outfile' instead of
# dumping it to stdout. Generated patches automatically being tagged with
# "FreeBSD" cvs id.
#
def gendiff(path, wrksrc, outfile = ''):
	fullpath = os.path.join(wrksrc, path)
	if not os.path.isfile(fullpath):
		raise IOError(errno.ENOENT, fullpath)
		# Not reached #

	cmdline = ''
	if os.path.isfile(fullpath + Vars.DIFF_SUFX):		# Normal diff
		path_orig = path + Vars.DIFF_SUFX
		cmdline = '%s %s %s %s' % (Vars.DIFF_CMD, Vars.DIFF_ARGS, path_orig, path)
	elif os.path.isfile(fullpath + Vars.RCSDIFF_SUFX):	# RCS diff
		path_orig = path
		cmdline = '%s %s %s' % (Vars.RCSDIFF_CMD, Vars.DIFF_ARGS, path)
	else:							# New file
		path_orig = Vars.DEV_NULL
		cmdline = '%s %s %s %s' % (Vars.DIFF_CMD, Vars.DIFF_ARGS, path_orig, path)

	savedir = os.getcwd()
	os.chdir(wrksrc)
	devnull = open('/dev/null', 'a')
	pipe = subprocess.Popen(cmdline, shell = True, stdin = subprocess.PIPE, \
            stdout = subprocess.PIPE, stderr = devnull, close_fds = True)
	outbuf = [x.decode() for x in pipe.stdout.readlines()]
	exitval = pipe.wait()
	if exitval == 0:    # No differences were found
		retval = False
		retmsg = 'no differences found between original and current ' \
			  'version of "%s"' % fullpath
	elif exitval == 1:  # Some differences  were  found
		if (outfile != ''):
			outbuf[0] = '--- %s\n' % path_orig
			outbuf[1] = '+++ %s\n' % path
			open(outfile, 'w').writelines(outbuf)
		else:
			sys.stdout.writelines(outbuf)
		retval = True
		retmsg = ''
	else:               # Error occurred
		raise ECmdError('"%s"' % cmdline, \
		  'external command returned non-zero error code')
		# Not reached #

	os.chdir(savedir)
	return (retval, retmsg)


#
# Automatically generate a name for a patch based on its path relative to
# wrksrc. Use simple scheme to ensure 1-to-1 mapping between path and
# patchname - replace all '_' with '__' and all '/' with '_'.
#
def makepatchname(path, patchdir = ''):
	SRS = Vars.SLASH_REPL_SYMBOL
	retval = Vars.PATCH_PREFIX + \
	  path.replace(SRS, SRS + SRS).replace('/', SRS)
	retval = os.path.join(patchdir, retval)
	return retval


#
# Write a specified message to stderr.
#
def write_msg(message):
	if isStr(message):
		message = message,
	sys.stderr.writelines(message)


#
# Print specified message to stdout and ask user [y/N]?. Optionally allow
# specify default answer, i.e. return value if user typed only <cr>
#
def query_yn(message, default = False):
	while True:
		if default == True:
			yn = 'Y/n'
		elif default == False:
			yn = 'y/N'
		else:
			yn = 'Y/N'

		reply = compat_raw_input('%s [%s]: ' % (message, yn))

		if reply == 'y' or reply == 'Y':
			return True
		elif reply == 'n' or reply == 'N':
			return False
		elif reply == '' and default in (True, False):
			return default
		print('Wrong answer "%s", please try again' % reply)
	return default


#
# Print optional message and usage information and exit with specified exit
# code.
#
def usage(code, msg = ''):
	myname = os.path.basename(sys.argv[0])
	write_msg((str(msg), """
Usage: %s [-afi] file ...
       %s -u [-i] [patchfile|patchdir ...]
""" % (myname, myname)))
	sys.exit(code)


#
# Simple custom exception
#
class MyError(Exception):
	msg = 'error'

	def __init__(self, file, msg=''):
		self.file = file
		if msg != '':
			self.msg = msg

	def __str__(self):
			return '%s: %s' % (self.file, self.msg)


#
# Error parsing patchfile
#
class PatchError(MyError):
	msg = 'corrupt patchfile, or not patchfile at all'


#
# Error executing external command
#
class ECmdError(MyError):
	pass


#
# Error getting value of makefile variable
#
class MakeVarError(MyError):
	def __init__(self, file, makevar, msg=''):
		self.file = file
		if msg != '':
			self.msg = msg
		else:
			self.msg = 'can\'t get %s value' % makevar


#
# Error locating portdir
#
class LocatePDirError(MyError):
	msg = 'can\'t locate portdir'


class Patch:
	fullpath = ''
	minus3file = ''
	plus3file = ''
	wrksrc = ''
	patchmtime = 0
	targetmtime = 0

	def __init__(self, path, wrksrc):
		MINUS3_DELIM = '--- '
		PLUS3_DELIM = '+++ '

		path = os.path.abspath(path)
		if not os.path.isfile(path):
			raise IOError(errno.ENOENT, path)
			# Not reached #

		self.fullpath = path
		filedes = open(path)

		for line in filedes.readlines():
			if self.minus3file == '':
				if line[:len(MINUS3_DELIM)] == MINUS3_DELIM:
					lineparts = line.split()
					try:
						self.minus3file = lineparts[1]
					except IndexError:
						raise PatchError(path)
						# Not reached #
					continue
			elif line[:len(PLUS3_DELIM)] == PLUS3_DELIM:
				lineparts = line.split()
				try:
					self.plus3file = lineparts[1]
				except IndexError:
					raise PatchError(path)
					# Not reached #
				break

		filedes.close()

		if self.minus3file == '' or self.plus3file == '':
			raise PatchError(path)
			# Not reached #

		self.wrksrc = os.path.abspath(wrksrc)
		self.patchmtime = os.path.getmtime(self.fullpath)
		plus3file = os.path.join(self.wrksrc, self.plus3file)
		if os.path.isfile(plus3file):
			self.targetmtime = os.path.getmtime(plus3file)
		else:
			self.targetmtime = 0

	def update(self, patch_cookiemtime = 0, ignoremtime = False):
		targetfile = os.path.join(self.wrksrc, self.plus3file)
		if not os.path.isfile(targetfile):
			raise IOError(errno.ENOENT, targetfile)
			# Not reached #

		patchdir = os.path.dirname(self.fullpath)
		if not os.path.isdir(patchdir):
			os.mkdir(patchdir)

		if ignoremtime == True or self.patchmtime == 0 or \
		  self.targetmtime == 0 or \
		  (self.patchmtime < self.targetmtime and \
		  patch_cookiemtime < self.targetmtime):
			retval = gendiff(self.plus3file, self.wrksrc, self.fullpath)
			if retval[0] == True:
				self.patchmtime = os.path.getmtime(self.fullpath)
		else:
			retval = (False, 'patch is already up to date')
		return retval


class NewPatch(Patch):
	def __init__(self, patchdir, wrksrc, relpath):
		self.fullpath = makepatchname(relpath, os.path.abspath(patchdir))
		self.wrksrc = os.path.abspath(wrksrc)
		self.plus3file = relpath
		self.minus3file = relpath
		self.patchmtime = 0
		plus3file = os.path.join(self.wrksrc, self.plus3file)
		if os.path.isfile(plus3file):
			self.targetmtime = os.path.getmtime(plus3file)
		else:
			self.targetmtime = 0


class PatchesCollection:
	patches = {}

	def __init__(self):
		self.patches = {}
		pass

	def adddir(self, patchdir, wrksrc):
		if not os.path.isdir(patchdir):
			raise IOError(errno.ENOENT, patchdir)
			# Not reached #

		for filename in glob.glob(os.path.join(patchdir, Vars.PATCH_PREFIX + '*')):
			for sufx in Vars.PATCH_IGN_SUFX:
				if filename[-len(sufx):] == sufx:
					write_msg('WARNING: patchfile "%s" ignored\n' % filename)
					break
			else:
				self.addpatchfile(filename, wrksrc)

	def addpatchfile(self, path, wrksrc):
		path = os.path.abspath(path)
		if path not in self.patches:
			self.addpatchobj(Patch(path, wrksrc))

	def addpatchobj(self, patchobj):
		self.patches[patchobj.fullpath] = patchobj

	def lookupbyname(self, path):
		path = os.path.abspath(path)
		if path in self.patches:
			return self.patches[path]
		return None

	def lookupbytarget(self, wrksrc, relpath):
		wrksrc = os.path.abspath(wrksrc)
		for patch in self.patches.values():
			if wrksrc == patch.wrksrc and relpath == patch.plus3file:
				return patch
		return None

	def getpatchobjs(self):
		return self.patches.values()


#
# Resolve all symbolic links in the given path to a file
#
def truepath(path):
	if not os.path.isfile(path):
		raise IOError(errno.ENOENT, path)

	result = ''
	while len(path) > 0:
		path, lastcomp = os.path.split(path)
		if len(lastcomp) == 0:
			lastcomp = path
			path = ''
		result = os.path.join(lastcomp, result)
		if len(path) == 0:
			break
		if os.path.islink(path):
			linkto = os.path.normpath(os.readlink(path))
			if linkto[0] != '/':
				path = os.path.join(path, linkto)
			else:
				path = linkto
	return result[:-1]


def main():
	try:
		opts, args = getopt.getopt(sys.argv[1:], 'afui')
	except getopt.GetoptError as msg:
		usage(2, msg)

	automatic = False
	force = False
	mode = generate
	ignoremtime = False

	for o, a in opts:
		if o == '-a':
			automatic = True
		elif o == '-f':
			force = True
		elif o == '-u':
			mode = update
		elif o == '-i':
			ignoremtime = True
		else:
			usage(2)

	# Allow user to override internal constants
	for varname in dir(Vars):
		if varname[:2] == '__' and varname[-2:] == '__':
			continue
		try:
			value = os.environ['PT_' + varname]
			setattr(Vars, varname, value)
		except KeyError:
			pass

	mode(args, automatic, force, ignoremtime)

	sys.exit(0)


#
# Display a diff or generate patchfile for the files pointed out by args.
#
def generate(args, automatic, force, ignoremtime):
	if len(args) == 0:
		usage(2, "ERROR: no input files specified")

	patches = PatchesCollection()

	for filepath in args:
		for suf in Vars.RCSDIFF_SUFX, Vars.DIFF_SUFX:
			if filepath.endswith(suf):
				filepath = filepath[:-len(suf)]
				break
		if not os.path.isfile(filepath):
			raise IOError(errno.ENOENT, filepath)
			# Not reached #

		filepath = truepath(filepath)

		wrkdirprefix = querymakevar('WRKDIRPREFIX', Vars.ETC_MAKE_CONF, False)
		portdir = locateportdir(os.path.dirname(filepath), wrkdirprefix, True)
		wrksrc = querymakevar('WRKSRC', portdir, True)

		relpath = getrelpath(filepath, wrksrc)

		if automatic:
			patchdir = querymakevar('PATCHDIR', portdir, True)

			if os.path.isdir(patchdir):
				patches.adddir(patchdir, wrksrc)

			extra_patches = querymakevar('EXTRA_PATCHES', portdir, False)
			for extra_patch in extra_patches.split():
				if os.path.isfile(extra_patch):
					patches.addpatchfile(extra_patch, wrksrc)

			patchobj = patches.lookupbytarget(wrksrc, relpath)
			if patchobj == None:
				patchobj = NewPatch(patchdir, wrksrc, relpath)
				patches.addpatchobj(patchobj)

			if not force and os.path.exists(patchobj.fullpath) and \
			  os.path.getsize(patchobj.fullpath) > 0:
				try:
					retval = query_yn('Target patchfile "%s" already ' \
					  'exists, do you want to  replace it?' % \
					  os.path.basename(patchobj.fullpath))
				except KeyboardInterrupt:
					sys.exit('\nAction aborted')
					# Not reached #
				if retval == False:
					continue

			write_msg('Generating patchfile: %s...' % \
			  os.path.basename(patchobj.fullpath))

			try:
				retval = None
				retval = patchobj.update(ignoremtime = ignoremtime)
			finally:
				# Following tricky magic intended to let us append \n even if
				# we are going to die due to unhandled exception
				if retval == None:
					write_msg('OUCH!\n')

			if retval[0] == False:
				write_msg('skipped (%s)\n' % retval[1])
			else:
				write_msg('ok\n')

		else:	# automatic != True
			retval = gendiff(relpath, wrksrc)
			if retval[0] == False:
				write_msg('WARNING: %s\n' % retval[1])


#
# Atomatically update all patches pointed by args (may be individual
# patchfiles, patchdirs or any directories in a portdirs). If directory argument
# is encountered, all patches that belong to the port are updated. If no
# arguments are supplied - current directory is assumed.
#
# The procedure honours last modification times of the patchfile, file from
# which diff to be generated and `EXTRACT_COOKIE' file (usually
# ${WRKDIR}/.extract_cookie) to update only those patches that are really need
# to be updated.
#
def update(args, automatic, force, ignoremtime):
	if len(args) == 0:
		args = './',

	for path in args:
		if not os.path.exists(path):
			raise IOError(errno.ENOENT, path)
			# Not reached #

		patches = PatchesCollection()

		if os.path.isdir(path):
			for wrkdirprefix in (querymakevar('WRKDIRPREFIX', \
              Vars.ETC_MAKE_CONF, False), ''):
				portdir = locateportdir(path, wrkdirprefix, False)
				if portdir != '':
					break
			if portdir == '':
				raise LocatePDirError(os.path.abspath(path))
				# Not reached #

			wrksrc = querymakevar('WRKSRC', portdir, True)
			patchdir = querymakevar('PATCHDIR', portdir, True)

			if os.path.isdir(patchdir):
				patches.adddir(patchdir, wrksrc)
			else:
				continue

		elif os.path.isfile(path):
			portdir = locateportdir(os.path.dirname(path), '' , True)
			wrksrc = querymakevar('WRKSRC', portdir, True)
			patches.addpatchfile(path, wrksrc)

		patch_cookie = querymakevar('PATCH_COOKIE', portdir, True)
		if os.path.isfile(patch_cookie):
			patch_cookiemtime = os.path.getmtime(patch_cookie)
		else:
			patch_cookiemtime = 0

		for patchobj in patches.getpatchobjs():
			write_msg('Updating patchfile: %s...' % \
			  os.path.basename(patchobj.fullpath))

			try:
				retval = None
				retval = patchobj.update(patch_cookiemtime, \
				  ignoremtime)
			finally:
				if retval == None:
					write_msg('OUCH!\n')

			if retval[0] == False:
				write_msg('skipped (%s)\n' % retval[1])
			else:
				write_msg('ok\n')


if __name__ == '__main__':
	try:
		main()
	except (PatchError, ECmdError, MakeVarError, LocatePDirError) as msg:
		sys.exit('ERROR: ' + str(msg))
	except IOError as ex:
		code, msg = ex
		sys.exit('ERROR: %s: %s' % (str(msg), os.strerror(code)))




-- Fix tests for FreeBSD (don't delete temporary files on close, fix np.int).

-- Fix np.int usage for NumPy 2.0+ compatibility.

--- tests/test_connection.py.orig	2021-10-05 14:19:26 UTC
+++ tests/test_connection.py
@@ -8,16 +8,16 @@ class LoomConnectionTests(TestCase):
 
 class LoomConnectionTests(TestCase):
     def setUp(self) -> None:
-        self.file = NamedTemporaryFile(suffix=".loom")
+        self.file = NamedTemporaryFile(suffix=".loom", delete=False)
         self.file.close()
         loompy.create(
             self.file.name,
             np.random.random((5, 5)),
             row_attrs={
-                "key": np.fromiter(range(5), dtype=np.int)
+                "key": np.fromiter(range(5), dtype=int)
             },
             col_attrs={
-                "key": np.fromiter(range(5), dtype=np.int)
+                "key": np.fromiter(range(5), dtype=int)
             })
 
     def tearDown(self) -> None:
